"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.XRPImplementation = void 0;
const axios_1 = __importDefault(require("axios"));
const __1 = require("..");
const axios_rate_limit_1 = __importDefault(require("../axios-rate-limiter/axios-rate-limit"));
const StatusBase_1 = require("../base-objects/StatusBase");
const globalSettings_1 = require("../global-settings/globalSettings");
const types_1 = require("../types");
const constants_1 = require("../utils/constants");
const errors_1 = require("../utils/errors");
const trace_1 = require("../utils/trace");
const utils_1 = require("../utils/utils");
const xrpUtils_1 = require("../utils/xrpUtils");
const DEFAULT_TIMEOUT = 15000;
const DEFAULT_RATE_LIMIT_OPTIONS = {
    maxRPS: 5,
};
let XRPImplementation = class XRPImplementation {
    constructor(createConfig) {
        var _a;
        const createAxiosConfig = {
            baseURL: createConfig.url,
            timeout: ((_a = createConfig.rateLimitOptions) === null || _a === void 0 ? void 0 : _a.timeoutMs) || DEFAULT_TIMEOUT,
            headers: { "Content-Type": "application/json" },
            validateStatus: function (status) {
                return (status >= 200 && status < 300) || status == 500;
            },
        };
        if (createConfig.username && createConfig.password) {
            createAxiosConfig.auth = {
                username: createConfig.username,
                password: createConfig.password,
            };
        }
        let client = axios_1.default.create(createAxiosConfig);
        this.client = (0, axios_rate_limit_1.default)(client, Object.assign(Object.assign({}, DEFAULT_RATE_LIMIT_OPTIONS), createConfig.rateLimitOptions));
        this.inRegTest = createConfig.inRegTest || false;
        this.chainType = types_1.ChainType.XRP;
    }
    /**
     * get NodeStatus object with information about node used to connect to underlying chain
     * @external_docs https://xrpl.org/server_state.html
     */
    getNodeStatus() {
        return __awaiter(this, void 0, void 0, function* () {
            let res = yield this.client.post("", {
                method: "server_state",
                params: [],
            });
            (0, xrpUtils_1.xrp_ensure_data)(res.data);
            return new StatusBase_1.XrpNodeStatus(res.data);
        });
    }
    /**
     * Get the height of the block from which the underlying node holds the full history
     * @returns the block height of the first block in latest joined block set in node memory
     */
    getBottomBlockHeight() {
        return __awaiter(this, void 0, void 0, function* () {
            let res = yield this.client.post("", {
                method: "server_state",
                params: [],
            });
            (0, xrpUtils_1.xrp_ensure_data)(res.data);
            try {
                const Ledgers = res.data.result.state.complete_ledgers.split(",").sort();
                return parseInt(Ledgers[Ledgers.length - 1].split("-")[0]);
            }
            catch (e) {
                throw new errors_1.mccOutsideError(e);
            }
        });
    }
    ///////////////////////////////////////////////////////////////////////////////////////
    // Block methods //////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////
    getBlock(blockNumberOrHash) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof blockNumberOrHash === "string") {
                if (constants_1.PREFIXED_STD_BLOCK_HASH_REGEX.test(blockNumberOrHash)) {
                    blockNumberOrHash = (0, utils_1.unPrefix0x)(blockNumberOrHash);
                }
            }
            try {
                globalSettings_1.mccSettings.loggingCallback(`block number: ${blockNumberOrHash} `);
                let res = yield this.client.post("", {
                    method: "ledger",
                    params: [
                        {
                            ledger_index: blockNumberOrHash,
                            transactions: true,
                            expand: true,
                            binary: false,
                        },
                    ],
                });
                (0, xrpUtils_1.xrp_ensure_data)(res.data);
                return new __1.XrpBlock(res.data);
            }
            catch (e) {
                if (((_a = e === null || e === void 0 ? void 0 : e.result) === null || _a === void 0 ? void 0 : _a.error) === "lgrNotFound") {
                    throw new errors_1.mccError(errors_1.mccErrorCode.InvalidBlock);
                }
                if (((_b = e === null || e === void 0 ? void 0 : e.response) === null || _b === void 0 ? void 0 : _b.status) === 400) {
                    throw new errors_1.mccError(errors_1.mccErrorCode.InvalidBlock);
                }
                throw e;
            }
        });
    }
    getBlockHeight() {
        return __awaiter(this, void 0, void 0, function* () {
            let res = yield this.client.post("", {
                method: "ledger_closed",
                params: [{}],
            });
            (0, xrpUtils_1.xrp_ensure_data)(res.data);
            return res.data.result.ledger_index;
        });
    }
    ///////////////////////////////////////////////////////////////////////////////////////
    // Transaction methods ////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////
    getTransaction(txId, options) {
        return __awaiter(this, void 0, void 0, function* () {
            if (constants_1.PREFIXED_STD_TXID_REGEX.test(txId)) {
                txId = (0, utils_1.unPrefix0x)(txId);
            }
            const binary = (options === null || options === void 0 ? void 0 : options.binary) || false;
            const min_block = (options === null || options === void 0 ? void 0 : options.min_block) || undefined;
            const max_block = (options === null || options === void 0 ? void 0 : options.max_block) || undefined;
            let params = {
                transaction: txId,
                binary: binary,
            };
            if (min_block !== null && min_block !== null) {
                params.min_ledger = min_block;
                params.max_ledger = max_block;
            }
            try {
                let res = yield this.client.post("", {
                    method: "tx",
                    params: [params],
                });
                (0, xrpUtils_1.xrp_ensure_data)(res.data);
                return new __1.XrpTransaction(res.data);
            }
            catch (e) {
                if (e.error === "txnNotFound") {
                    throw new errors_1.mccError(errors_1.mccErrorCode.InvalidTransaction);
                }
                throw e;
            }
        });
    }
    ///////////////////////////////////////////////////////////////////////////////////////
    // Client specific methods ////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////
    /**
     *
     * @param account A unique identifier for the account, most commonly the account's Address.
     * @param upperBound either blockHash or block number for the upper bound (The information does not contain any changes from ledger versions newer than this one.)
     * @returns
     */
    getAccountInfo(account, upperBound = "current") {
        return __awaiter(this, void 0, void 0, function* () {
            const params = {
                account: account,
                signer_lists: true,
            };
            if (typeof upperBound === "number") {
                params.ledger_index = upperBound;
            }
            else if (upperBound === "current") {
                params.ledger_index = upperBound;
            }
            else if (typeof upperBound === "string") {
                params.ledger_hash = upperBound;
            }
            else {
                globalSettings_1.mccSettings.errorCallback(upperBound, "Invalid upperBound parameter");
            }
            // AccountInfoRequest
            globalSettings_1.mccSettings.loggingCallback("Call Params");
            globalSettings_1.mccSettings.loggingCallback(JSON.stringify(params));
            let res = yield this.client.post("", {
                method: "account_info",
                params: [params],
            });
            (0, xrpUtils_1.xrp_ensure_data)(res.data);
            return res.data;
        });
    }
    getAccountTransactions(account, lowerBound = -1, upperBound = -1) {
        return __awaiter(this, void 0, void 0, function* () {
            const params = {
                account: account,
            };
            params.ledger_index_min = lowerBound;
            params.ledger_index_max = upperBound;
            globalSettings_1.mccSettings.loggingCallback(JSON.stringify(params));
            let res = yield this.client.post("", {
                method: "account_tx",
                params: [params],
            });
            (0, xrpUtils_1.xrp_ensure_data)(res.data);
            return res.data;
        });
    }
};
XRPImplementation = __decorate([
    (0, trace_1.Trace)()
], XRPImplementation);
exports.XRPImplementation = XRPImplementation;
//# sourceMappingURL=XrpRpcImplementation.js.map