/// <reference types="node" />
import { IAlgoHexAddress } from "../types/algoTypes";
import algosdk from "algosdk";
export declare const INVALIDADDRESERROR: (address: string) => Error;
export declare const INVALIDPUBKEYPAIRERROR: (algoKeyPair: IAlgoHexAddress) => Error;
export declare function hexToBytes(hex: string): Uint8Array;
export declare function bytesToHex(bytes: Buffer | Uint8Array): string;
/**
 * Algos base32 to hex string
 * @param data
 * @returns
 */
export declare function base32ToHex(data: string): string;
export declare function hexToBase32(hex: string | Uint8Array): any;
/**
 * For note decoding
 * @param rawdata
 * @returns
 */
export declare function base64ToHex(rawdata: string): string;
export declare function hexToBase64(hex: string | Uint8Array): string;
/**
 *
 * @dev used for transaction note parsing
 * @param rawdata
 * @returns
 */
export declare function base64ToText(rawdata: string): string;
export declare function addressToHex(address: string): IAlgoHexAddress;
export declare function hexToAddress(algoKeyPair: IAlgoHexAddress): string;
export declare function b32AddressTob32CAddress(address: string): void;
/**
 * buffer address to buffer address with checksum
 */
export declare function bufAddToCBufAdd(address: Buffer): Buffer;
export declare function txIdToHex(txid: string): string;
export declare function txIdToHexNo0x(txid: string): string;
export declare function algo_ensure_data(data: any): void;
export declare function algo_check_expect_block_out_of_range(data: any): boolean;
export declare function algo_check_expect_empty(data: any): boolean;
export declare function mpEncode(obj: Record<string | number | symbol, any>): Uint8Array;
export declare function mpDecode(buffer: ArrayLike<number>): unknown;
export declare class StateDelta {
    action: number;
    bytes: Uint8Array;
    uint: number | undefined;
    static fromMsgp(state_delta: any): StateDelta;
    get_obj_for_encoding(): any;
}
export declare class EvalDelta {
    global_delta: StateDelta[];
    local_deltas: {
        [key: number]: StateDelta[];
    };
    logs: string[];
    inner_txns: SignedTransactionWithAD[];
    constructor(o: {
        global_delta?: StateDelta[];
        local_deltas?: {
            [key: number]: StateDelta[];
        };
        logs?: string[];
        inner_txns?: SignedTransactionWithAD[];
    });
    static fromMsgp(delta: any): EvalDelta;
    get_obj_for_encoding(): any;
}
export declare class ApplyData {
    closing_amount: number;
    asset_closing_amount: number;
    sender_rewards: number;
    receiver_rewards: number;
    close_rewards: number;
    eval_delta: EvalDelta | undefined;
    config_asset: number;
    application_id: number;
    constructor(o: {
        closing_amount?: 0;
        asset_closing_amount?: 0;
        sender_rewards?: 0;
        receiver_rewards?: 0;
        close_rewards?: 0;
        eval_delta?: undefined;
        config_asset?: 0;
        application_id?: 0;
    });
    static fromMsgp(apply_data: any): ApplyData;
    get_obj_for_encoding(): any;
}
export declare class SignedTransactionWithAD {
    txn: algosdk.SignedTransaction;
    apply_data: ApplyData | undefined;
    constructor(gh: Buffer, gen: string, stib: any);
    get_obj_for_encoding(): any;
    hash(): Uint8Array;
}
export declare function hasher(data: Uint8Array): Uint8Array;
export declare function concatArrays(...arrs: ArrayLike<number>[]): Uint8Array;
//# sourceMappingURL=algoUtils.d.ts.map