"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.concatArrays = exports.hasher = exports.SignedTransactionWithAD = exports.ApplyData = exports.EvalDelta = exports.StateDelta = exports.mpDecode = exports.mpEncode = exports.algo_check_expect_empty = exports.algo_check_expect_block_out_of_range = exports.algo_ensure_data = exports.txIdToHexNo0x = exports.txIdToHex = exports.bufAddToCBufAdd = exports.b32AddressTob32CAddress = exports.hexToAddress = exports.addressToHex = exports.base64ToText = exports.hexToBase64 = exports.base64ToHex = exports.hexToBase32 = exports.base32ToHex = exports.bytesToHex = exports.hexToBytes = exports.INVALIDPUBKEYPAIRERROR = exports.INVALIDADDRESERROR = void 0;
const utils_1 = require("./utils");
const base32 = require("base32.js");
const sha512_256 = require("js-sha512").sha512_256;
const msgpack = __importStar(require("algo-msgpack-with-bigint"));
const algosdk_1 = __importDefault(require("algosdk"));
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////// Errors //////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
const INVALIDADDRESERROR = (address) => {
    return new Error(`Invalid address: ${address}`);
};
exports.INVALIDADDRESERROR = INVALIDADDRESERROR;
const INVALIDPUBKEYPAIRERROR = (algoKeyPair) => {
    return new Error(`Invalid keypair: ${algoKeyPair.publicKey} -  ${algoKeyPair.checksum}`);
};
exports.INVALIDPUBKEYPAIRERROR = INVALIDPUBKEYPAIRERROR;
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////// Code ////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////
// Bytes <-> Hex //
///////////////////
// Convert a hex string to a byte array
function hexToBytes(hex) {
    let bytes = [];
    for (let c = 0; c < hex.length; c += 2)
        bytes.push(parseInt(hex.substr(c, 2), 16));
    return new Uint8Array(bytes);
}
exports.hexToBytes = hexToBytes;
// Convert a byte array to a hex string
function bytesToHex(bytes) {
    // should be buffer
    for (var hex = [], i = 0; i < bytes.length; i++) {
        var current = bytes[i] < 0 ? bytes[i] + 256 : bytes[i];
        hex.push((current >>> 4).toString(16));
        hex.push((current & 0xf).toString(16));
    }
    return hex.join("");
}
exports.bytesToHex = bytesToHex;
////////////////////
// Base32 <-> Hex //
////////////////////
/**
 * Algos base32 to hex string
 * @param data
 * @returns
 */
function base32ToHex(data) {
    // old base 32 decoding
    const decoder = new base32.Decoder({ type: "rfc4648" });
    let out = decoder.write(data).finalize();
    return out.toString("hex");
}
exports.base32ToHex = base32ToHex;
function hexToBase32(hex) {
    // old base 32 encoding
    const encoder = new base32.Encoder({ type: "rfc4648" });
    if (typeof hex === "string") {
        let bufHex = Buffer.from(hex, 'hex');
        let base32Out = encoder.write(bufHex).finalize();
        return base32Out;
    }
    else {
        let bufHex = Buffer.from(bytesToHex(hex), 'hex');
        let base32Out = encoder.write(bufHex).finalize();
        return base32Out;
    }
}
exports.hexToBase32 = hexToBase32;
////////////////////
// Base64 <-> Hex //
////////////////////
/**
 * For note decoding
 * @param rawdata
 * @returns
 */
function base64ToHex(rawdata) {
    let data = Buffer.from(rawdata, "base64").toString("binary");
    let hexData = "";
    for (let i = 0; i < data.length; i++) {
        const hex = data.charCodeAt(i).toString(16);
        hexData += hex.length === 2 ? hex : "0" + hex;
    }
    return hexData;
}
exports.base64ToHex = base64ToHex;
function hexToBase64(hex) {
    if (typeof hex === "string") {
        return Buffer.from(hex, "hex").toString("base64");
    }
    else {
        return Buffer.from(bytesToHex(hex), "hex").toString("base64");
    }
}
exports.hexToBase64 = hexToBase64;
/////////////////////
// Base64 <-> Text //
/////////////////////
/**
 *
 * @dev used for transaction note parsing
 * @param rawdata
 * @returns
 */
function base64ToText(rawdata) {
    let buff = Buffer.from(rawdata, "base64");
    return buff.toString("ascii");
}
exports.base64ToText = base64ToText;
//////////////////////////
// Algo address <-> Hex //
//////////////////////////
function addressToHex(address) {
    if (!algosdk_1.default.isValidAddress(address)) {
        throw (0, exports.INVALIDADDRESERROR)(address);
    }
    let dad = algosdk_1.default.decodeAddress(address);
    const publicKey = Buffer.from(dad.publicKey).toString("hex");
    const checksum = Buffer.from(dad.checksum).toString("hex");
    return {
        publicKey: (0, utils_1.prefix0x)(publicKey),
        checksum: (0, utils_1.prefix0x)(checksum),
    };
}
exports.addressToHex = addressToHex;
function hexToAddress(algoKeyPair) {
    let conc = (0, utils_1.unPrefix0x)(algoKeyPair.publicKey) + (0, utils_1.unPrefix0x)(algoKeyPair.checksum);
    let concBytes = hexToBytes(conc);
    const address = hexToBase32(concBytes);
    if (!algosdk_1.default.isValidAddress(address)) {
        throw (0, exports.INVALIDPUBKEYPAIRERROR)(algoKeyPair);
    }
    return address;
}
exports.hexToAddress = hexToAddress;
///////////////////////////////////////
// Algo address <-> address+checksum //
///////////////////////////////////////
// address checksum
// https://emn178.github.io/online-tools/sha512_256.html hash SHA512/256
// https://github.com/algorand/go-algorand-sdk/blob/develop/types/address.go
function b32AddressTob32CAddress(address) { }
exports.b32AddressTob32CAddress = b32AddressTob32CAddress;
/**
 * buffer address to buffer address with checksum
 */
function bufAddToCBufAdd(address) {
    if (address.length === 32) {
        const checksumString = sha512_256.arrayBuffer(address);
        const checksum = checksumString.slice(28, 32);
        return Buffer.concat([address, Buffer.from(checksum)], 36);
    }
    else if (address.length === 36) {
        return address;
    }
    else {
        throw new Error(`Not a valid address buffer: ${address}`);
    }
}
exports.bufAddToCBufAdd = bufAddToCBufAdd;
/////////////////////////////////
// Algo transaction id <-> Hex //
/////////////////////////////////
function txIdToHex(txid) {
    return (0, utils_1.prefix0x)(base32ToHex(txid));
}
exports.txIdToHex = txIdToHex;
function txIdToHexNo0x(txid) {
    return base32ToHex(txid);
}
exports.txIdToHexNo0x = txIdToHexNo0x;
////////////////////////////
//// MCC Error handling ////
////////////////////////////
function algo_ensure_data(data) {
    const error_codes = [400, 401, 404, 500, 503];
    if (error_codes.includes(data.status)) {
        throw (0, utils_1.MccError)(data);
    }
}
exports.algo_ensure_data = algo_ensure_data;
function algo_check_expect_block_out_of_range(data) {
    if (data.status === 404) {
        return true;
    }
    else {
        return false;
    }
}
exports.algo_check_expect_block_out_of_range = algo_check_expect_block_out_of_range;
function algo_check_expect_empty(data) {
    if (data.status === 404) {
        return true;
    }
    else {
        return false;
    }
}
exports.algo_check_expect_empty = algo_check_expect_empty;
////////////////////////////////////////////
//// Message Pack encoding and decoding ////
////////////////////////////////////////////
function mpEncode(obj) {
    // enable the canonical option
    const options = { sortKeys: true };
    return msgpack.encode(obj, options);
}
exports.mpEncode = mpEncode;
function mpDecode(buffer) {
    return msgpack.decode(buffer);
}
exports.mpDecode = mpDecode;
////////////////////////////////////////////
//// Light (Non-Archival) block updates ////
////////////////////////////////////////////
class StateDelta {
    constructor() {
        this.action = 0;
        this.bytes = new Uint8Array();
        this.uint = undefined;
    }
    static fromMsgp(state_delta) {
        const sd = new StateDelta();
        if ("at" in state_delta)
            sd.action = state_delta["at"];
        if ("bs" in state_delta)
            sd.bytes = state_delta["bs"];
        if ("ui" in state_delta)
            sd.uint = state_delta["ui"];
        return sd;
    }
    get_obj_for_encoding() {
        var _a;
        const obj = {};
        if (this.action !== 0)
            obj["at"] = this.action;
        if (((_a = this.bytes) === null || _a === void 0 ? void 0 : _a.length) > 0)
            obj["bs"] = this.bytes;
        if (this.uint !== undefined)
            obj["ui"] = this.uint;
        return obj;
    }
}
exports.StateDelta = StateDelta;
class EvalDelta {
    constructor(o) {
        this.global_delta = [];
        this.local_deltas = {};
        this.logs = [];
        this.inner_txns = [];
    }
    static fromMsgp(delta) {
        const ed = new EvalDelta({});
        if ("gd" in delta) {
            try {
                for (const idx of delta["gd"]) {
                    ed.global_delta.push(StateDelta.fromMsgp(delta["gd"]));
                }
            }
            catch (e) {
                // TODO apparently not a part of txid -.-
            }
        }
        if ("ld" in delta) {
            try {
                for (const k of delta["ld"]) {
                    ed.local_deltas[k].push(StateDelta.fromMsgp(delta["ld"][k]));
                }
            }
            catch (e) {
                // TODO apparently not a part of txid -.-
            }
        }
        if ("itx" in delta) {
            try {
                for (const itxn of delta["itx"]) {
                    ed.inner_txns.push(new SignedTransactionWithAD(Buffer.from(""), "", itxn));
                }
            }
            catch (e) {
                // TODO apparently not a part of txid -.-
            }
        }
        if ("lg" in delta)
            ed.logs = delta["lg"];
        return ed;
    }
    get_obj_for_encoding() {
        var _a, _b, _c;
        const obj = {};
        if (this.global_delta.length > 0)
            obj["gd"] = this.global_delta.map((gd) => {
                return gd.get_obj_for_encoding();
            });
        if (((_a = Object.keys(this.local_deltas)) === null || _a === void 0 ? void 0 : _a.length) > 0)
            obj["ld"] = {};
        if (((_b = this.logs) === null || _b === void 0 ? void 0 : _b.length) > 0)
            obj["lg"] = this.logs;
        if (((_c = this.inner_txns) === null || _c === void 0 ? void 0 : _c.length) > 0)
            obj["itx"] = this.inner_txns.map((itxn) => {
                return itxn.get_obj_for_encoding();
            });
        return obj;
    }
}
exports.EvalDelta = EvalDelta;
class ApplyData {
    constructor(o) {
        this.closing_amount = 0;
        this.asset_closing_amount = 0;
        this.sender_rewards = 0;
        this.receiver_rewards = 0;
        this.close_rewards = 0;
        this.eval_delta = undefined;
        this.config_asset = 0;
        this.application_id = 0;
    }
    static fromMsgp(apply_data) {
        const ad = new ApplyData({});
        if ("ca" in apply_data)
            ad.closing_amount = apply_data["ca"];
        if ("aca" in apply_data)
            ad.asset_closing_amount = apply_data["aca"];
        if ("rs" in apply_data)
            ad.sender_rewards = apply_data["rs"];
        if ("rr" in apply_data)
            ad.receiver_rewards = apply_data["rr"];
        if ("rc" in apply_data)
            ad.close_rewards = apply_data["rc"];
        if ("caid" in apply_data)
            ad.config_asset = apply_data["caid"];
        if ("apid" in apply_data)
            ad.application_id = apply_data["apid"];
        if ("dt" in apply_data)
            ad.eval_delta = EvalDelta.fromMsgp(apply_data["dt"]);
        return ad;
    }
    get_obj_for_encoding() {
        const obj = {};
        if (this.closing_amount !== 0)
            obj["ca"] = this.closing_amount;
        if (this.asset_closing_amount !== 0)
            obj["aca"] = this.asset_closing_amount;
        if (this.sender_rewards !== 0)
            obj["rs"] = this.sender_rewards;
        if (this.receiver_rewards !== 0)
            obj["rr"] = this.receiver_rewards;
        if (this.close_rewards !== 0)
            obj["rc"] = this.close_rewards;
        if (this.config_asset !== 0)
            obj["caid"] = this.config_asset;
        if (this.application_id !== 0)
            obj["apid"] = this.application_id;
        if (this.eval_delta !== undefined)
            obj["dt"] = this.eval_delta.get_obj_for_encoding();
        return obj;
    }
}
exports.ApplyData = ApplyData;
class SignedTransactionWithAD {
    constructor(gh, gen, stib) {
        this.apply_data = undefined;
        const t = stib.txn;
        // Manually add gh/gen to construct a correct transaction object
        t.gh = gh;
        t.gen = gen;
        const stxn = {
            txn: algosdk_1.default.Transaction.from_obj_for_encoding(t),
        };
        if ("sig" in stib)
            stxn.sig = stib.sig;
        if ("lsig" in stib)
            stxn.lsig = stib.lsig;
        if ("msig" in stib)
            stxn.msig = stib.msig;
        if ("sgnr" in stib)
            stxn.sgnr = stib.sgnr;
        this.txn = stxn;
        this.apply_data = ApplyData.fromMsgp(stib);
    }
    get_obj_for_encoding() {
        var _a;
        const txn = this.txn.txn.get_obj_for_encoding();
        if (txn.gen !== "") {
            delete txn.gen;
            delete txn.gh;
        }
        const obj = Object.assign({ txn: txn }, (_a = this.apply_data) === null || _a === void 0 ? void 0 : _a.get_obj_for_encoding());
        if (this.txn.sig)
            obj["sig"] = this.txn.sig;
        if (this.txn.lsig)
            obj["lsig"] = this.txn.lsig;
        if (this.txn.msig)
            obj["msig"] = this.txn.msig;
        if (this.txn.sgnr)
            obj["sgnr"] = this.txn.sgnr;
        if (this.txn.txn.genesisID !== "")
            obj["hgi"] = true;
        return obj;
    }
    hash() {
        const obj = mpEncode(this.get_obj_for_encoding());
        return hasher(obj);
    }
}
exports.SignedTransactionWithAD = SignedTransactionWithAD;
// async function verifyProofHash(
//   block_number: number,
//   stxn: SignedTransactionWithAD
// ): Promise<boolean> {
//   const proof = await client.getProof(block_number, stxn.txn.txn.txID()).do();
//   const generated = Buffer.from(stxn.hash()).toString("base64");
//   return proof.stibhash == generated;
// }
function hasher(data) {
    const tohash = concatArrays(Buffer.from("STIB"), new Uint8Array(data));
    return new Uint8Array(sha512_256.array(tohash));
}
exports.hasher = hasher;
function concatArrays(...arrs) {
    var _a;
    const size = arrs.reduce((sum, arr) => sum + arr.length, 0);
    const c = new Uint8Array(size);
    let offset = 0;
    for (let i = 0; i < (arrs === null || arrs === void 0 ? void 0 : arrs.length); i++) {
        c.set(arrs[i], offset);
        offset += (_a = arrs[i]) === null || _a === void 0 ? void 0 : _a.length;
    }
    return c;
}
exports.concatArrays = concatArrays;
//# sourceMappingURL=algoUtils.js.map