"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.bytesToRippleAddress = exports.rippleAddressToBytes = exports.processFlagsOld2 = exports.processFlagsOld1 = exports.processFlags = exports.unixEpochToRippleTime = exports.rippleTimeToUnixEpoch = exports.xrp_ensure_data = void 0;
const types_1 = require("../types");
const constants_1 = require("./constants");
const utils_1 = require("./utils");
const XrpAddress = require("ripple-address-codec");
////////////////////////////
//// MCC Error handling ////
////////////////////////////
function xrp_ensure_data(data) {
    if (data.result.status === "error") {
        throw (0, utils_1.MccError)(data);
    }
}
exports.xrp_ensure_data = xrp_ensure_data;
function rippleTimeToUnixEpoch(timestamp) {
    return timestamp + constants_1.XRP_UTD;
}
exports.rippleTimeToUnixEpoch = rippleTimeToUnixEpoch;
function unixEpochToRippleTime(timestamp) {
    return timestamp - constants_1.XRP_UTD;
}
exports.unixEpochToRippleTime = unixEpochToRippleTime;
function processFlags(flag) {
    let altFlags = [];
    const flagPos = [16, 17, 18, 19, 20, 21, 22, 23, 24];
    for (let posFlag of flagPos) {
        if ((flag >> posFlag) % 2 === 1) {
            // @ts-ignore
            altFlags.push(types_1.PosToFlag[posFlag]);
        }
    }
    return altFlags;
}
exports.processFlags = processFlags;
function processFlagsOld1(flag) {
    const binFlag = flag.toString(2).padStart(8 * 4, "0");
    let altFlags = [];
    for (let posFlag of types_1.allHexFlags) {
        const posFlagBin = posFlag.toString(2).padStart(8 * 4, "0");
        if ((flag ^ posFlag).toString(2).padStart(8 * 4, "0")[posFlagBin.indexOf("1")] === "0") {
            // @ts-ignore
            altFlags.push(types_1.HexToFlag[posFlag]);
        }
    }
    return altFlags;
}
exports.processFlagsOld1 = processFlagsOld1;
function processFlagsOld2(flag) {
    const hexFlag = "0x" + flag.toString(16).padStart(8, "0");
    let Flags = [];
    switch (hexFlag[5]) {
        case "f": // 15
            Flags.push(types_1.HexToFlag[0x00010000]);
            Flags.push(types_1.HexToFlag[0x00020000]);
            Flags.push(types_1.HexToFlag[0x00040000]);
            Flags.push(types_1.HexToFlag[0x00080000]);
            break;
        case "e": // 14
            Flags.push(types_1.HexToFlag[0x00020000]);
            Flags.push(types_1.HexToFlag[0x00040000]);
            Flags.push(types_1.HexToFlag[0x00080000]);
            break;
        case "d": // 13
            Flags.push(types_1.HexToFlag[0x00010000]);
            Flags.push(types_1.HexToFlag[0x00040000]);
            Flags.push(types_1.HexToFlag[0x00080000]);
            break;
        case "c": // 12
            Flags.push(types_1.HexToFlag[0x00040000]);
            Flags.push(types_1.HexToFlag[0x00080000]);
            break;
        case "b": // 11
            Flags.push(types_1.HexToFlag[0x00010000]);
            Flags.push(types_1.HexToFlag[0x00020000]);
            Flags.push(types_1.HexToFlag[0x00080000]);
            break;
        case "a": // 10
            Flags.push(types_1.HexToFlag[0x00020000]);
            Flags.push(types_1.HexToFlag[0x00080000]);
            break;
        case "9":
            Flags.push(types_1.HexToFlag[0x00010000]);
            Flags.push(types_1.HexToFlag[0x00080000]);
            break;
        case "8":
            Flags.push(types_1.HexToFlag[0x00080000]);
            break;
        case "7":
            Flags.push(types_1.HexToFlag[0x00010000]);
            Flags.push(types_1.HexToFlag[0x00020000]);
            Flags.push(types_1.HexToFlag[0x00040000]);
            break;
        case "6":
            Flags.push(types_1.HexToFlag[0x00020000]);
            Flags.push(types_1.HexToFlag[0x00040000]);
            break;
        case "5":
            Flags.push(types_1.HexToFlag[0x00010000]);
            Flags.push(types_1.HexToFlag[0x00040000]);
            break;
        case "4":
            Flags.push(types_1.HexToFlag[0x00040000]);
            break;
        case "3":
            Flags.push(types_1.HexToFlag[0x00010000]);
            Flags.push(types_1.HexToFlag[0x00020000]);
            break;
        case "2":
            Flags.push(types_1.HexToFlag[0x00020000]);
            break;
        case "1":
            Flags.push(types_1.HexToFlag[0x00010000]);
            break;
    }
    switch (hexFlag[4]) {
        case "f": // 15
            Flags.push(types_1.HexToFlag[0x00100000]);
            Flags.push(types_1.HexToFlag[0x00200000]);
            Flags.push(types_1.HexToFlag[0x00400000]);
            Flags.push(types_1.HexToFlag[0x00800000]);
            break;
        case "e": // 14
            Flags.push(types_1.HexToFlag[0x00200000]);
            Flags.push(types_1.HexToFlag[0x00400000]);
            Flags.push(types_1.HexToFlag[0x00800000]);
            break;
        case "d": // 13
            Flags.push(types_1.HexToFlag[0x00100000]);
            Flags.push(types_1.HexToFlag[0x00400000]);
            Flags.push(types_1.HexToFlag[0x00800000]);
            break;
        case "c": // 12
            Flags.push(types_1.HexToFlag[0x00400000]);
            Flags.push(types_1.HexToFlag[0x00800000]);
            break;
        case "b": // 11
            Flags.push(types_1.HexToFlag[0x00100000]);
            Flags.push(types_1.HexToFlag[0x00200000]);
            Flags.push(types_1.HexToFlag[0x00800000]);
            break;
        case "a": // 10
            Flags.push(types_1.HexToFlag[0x00200000]);
            Flags.push(types_1.HexToFlag[0x00800000]);
            break;
        case "9":
            Flags.push(types_1.HexToFlag[0x00100000]);
            Flags.push(types_1.HexToFlag[0x00800000]);
            break;
        case "8":
            Flags.push(types_1.HexToFlag[0x00800000]);
            break;
        case "7":
            Flags.push(types_1.HexToFlag[0x00100000]);
            Flags.push(types_1.HexToFlag[0x00200000]);
            Flags.push(types_1.HexToFlag[0x00400000]);
            break;
        case "6":
            Flags.push(types_1.HexToFlag[0x00200000]);
            Flags.push(types_1.HexToFlag[0x00400000]);
            break;
        case "5":
            Flags.push(types_1.HexToFlag[0x00100000]);
            Flags.push(types_1.HexToFlag[0x00400000]);
            break;
        case "4":
            Flags.push(types_1.HexToFlag[0x00400000]);
            break;
        case "3":
            Flags.push(types_1.HexToFlag[0x00100000]);
            Flags.push(types_1.HexToFlag[0x00200000]);
            break;
        case "2":
            Flags.push(types_1.HexToFlag[0x00200000]);
            break;
        case "1":
            Flags.push(types_1.HexToFlag[0x00100000]);
            break;
    }
    switch (hexFlag[3]) {
        case "1":
            Flags.push(types_1.HexToFlag[0x01000000]);
            break;
    }
    return Flags;
}
exports.processFlagsOld2 = processFlagsOld2;
///////////////////////////
// Xrp address <-> Bytes //
///////////////////////////
function rippleAddressToBytes(address) {
    if (address.length > 0) {
        if (address[0] === "r") {
            return XrpAddress.decodeAccountID(address);
        }
        else {
            // it is a ripple x address
            const classic = XrpAddress.xAddressToClassicAddress(address);
            return XrpAddress.decodeAccountID(classic.classicAddress);
        }
    }
    return Buffer.from([0x00]);
}
exports.rippleAddressToBytes = rippleAddressToBytes;
function bytesToRippleAddress(byts) {
    if (byts.length === 20) {
        // it is a valid address
        return XrpAddress.encodeAccountID(byts);
    }
    throw new Error("Not a valid ripple address");
}
exports.bytesToRippleAddress = bytesToRippleAddress;
//# sourceMappingURL=xrpUtils.js.map