"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const src_1 = require("../../src");
const algoCreateConfig = {
    algod: {
        url: process.env.ALGO_ALGOD_URL || "",
        token: process.env.ALGO_ALGOD_TOKEN || "",
    },
};
describe(`Algo block processing`, () => __awaiter(void 0, void 0, void 0, function* () {
    describe("Classic block test ", function () {
        let MccClient;
        let block;
        const blockNumber = 21659776;
        before(function () {
            return __awaiter(this, void 0, void 0, function* () {
                MccClient = new src_1.MCC.ALGO(algoCreateConfig);
                block = yield MccClient.getBlock(blockNumber);
            });
        });
        it("Should get block", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(block).to.not.eq(undefined);
            });
        });
        it("Should get block number ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(block.number).to.eq(blockNumber);
            });
        });
        it("Should get block hash ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(block.blockHash).to.eq("rWPGk6t3twBulPE4lt7Yt9IHpgeOsqU1IlGAGJ5B74g=");
            });
        });
        it("Should get block hash base32 ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(block.blockHashBase32).to.eq("VVR4NE5LO63QA3UU6E4JNXWYW7JAPJQHR2ZKKNJCKGABRHSB56EA");
            });
        });
        it("Should get block hash base64 ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(block.blockHashBase64).to.eq("rWPGk6t3twBulPE4lt7Yt9IHpgeOsqU1IlGAGJ5B74g=");
            });
        });
        it("Should get block standard hash ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(block.stdBlockHash).to.eq("ad63c693ab77b7006e94f13896ded8b7d207a6078eb2a535225180189e41ef88");
            });
        });
        it("Should get block timestamp ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(block.unixTimestamp).to.eq(1655413908);
            });
        });
        it("Should get transaction ids ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(block.transactionIds.length).to.eq(69);
                // TODO at least check some txids
            });
        });
        it("Should get transaction standard ids ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(block.stdTransactionIds.length).to.eq(69);
            });
        });
        it("Should get transaction count ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(block.transactionCount).to.eq(69);
            });
        });
        it("Should get transaction objects (ALGO SPECIFIC) ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(block.transactions.length).to.eq(69);
            });
        });
    });
}));
//# sourceMappingURL=block.test.js.map