"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const src_1 = require("../../src");
const algoCreateConfig = {
    algod: {
        url: process.env.ALGO_ALGOD_URL || "",
        token: process.env.ALGO_ALGOD_TOKEN || "",
    },
};
describe("Algo transaction in block (Expect block 21_659_776 in node) ", function () {
    let MccClient;
    let block;
    before(function () {
        return __awaiter(this, void 0, void 0, function* () {
            MccClient = new src_1.MCC.ALGO(algoCreateConfig);
            block = yield MccClient.getBlock(21659776);
        });
    });
    describe("NativePayment transaction in block ", function () {
        let transaction;
        // https://algoexplorer.io/tx/TQA6RDIWO6FINMAJDQSMTKJTOCFO5CCXTWR2DWZJPZWDVAUZ4WQQ
        const txId = "TQA6RDIWO6FINMAJDQSMTKJTOCFO5CCXTWR2DWZJPZWDVAUZ4WQQ";
        before(function () {
            return __awaiter(this, void 0, void 0, function* () {
                for (let txOb of block.transactions) {
                    if (txOb.txid === txId) {
                        transaction = txOb;
                    }
                }
            });
        });
        it("Should find transaction in block ", function () {
            (0, chai_1.expect)(transaction).to.not.eq(undefined);
        });
        it("Should get transaction txid ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.txid).to.eq(txId);
            });
        });
        it("Should get standardized txid ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.stdTxid).to.eq("9c01e88d16778a86b0091c24c9a933708aee88579da3a1db297e6c3a8299e5a1");
            });
        });
        it("Should get transaction hash ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.hash).to.eq(txId);
            });
        });
        it("Should get transaction reference array ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.reference.length).to.eq(1);
            });
        });
        it("Should get standardized transaction reference ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.stdPaymentReference).to.eq("0x0000000000000000000000000000000000000000000000000000000000000000");
            });
        });
        it("Should get transaction timestamp ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.unixTimestamp).to.eq(1655413908);
            });
        });
        it("Should get source address ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.sourceAddresses.length).to.eq(1);
                (0, chai_1.expect)(transaction.sourceAddresses[0]).to.eq("XNFT36FUCFRR6CK675FW4BEBCCCOJ4HOSMGCN6J2W6ZMB34KM2ENTNQCP4");
            });
        });
        it("Should get receiving address ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.receivingAddresses.length).to.eq(1);
                (0, chai_1.expect)(transaction.receivingAddresses[0]).to.eq("5VA3MVA4SPRDSXK4X3YJJNCAO23SP4H57XLKZHRXZ3V7HRCF4BK4P66AJ4");
            });
        });
        it("Should get fee ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.fee.toNumber()).to.eq(1000);
            });
        });
        it("Should spend amount ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.spentAmounts.length).to.eq(1);
                (0, chai_1.expect)(transaction.spentAmounts[0].address).to.eq("XNFT36FUCFRR6CK675FW4BEBCCCOJ4HOSMGCN6J2W6ZMB34KM2ENTNQCP4");
                (0, chai_1.expect)(transaction.spentAmounts[0].amount.toNumber()).to.eq(251000);
            });
        });
        it("Should received amount ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.receivedAmounts.length).to.eq(1);
                (0, chai_1.expect)(transaction.receivedAmounts[0].address).to.eq("5VA3MVA4SPRDSXK4X3YJJNCAO23SP4H57XLKZHRXZ3V7HRCF4BK4P66AJ4");
                (0, chai_1.expect)(transaction.receivedAmounts[0].amount.toNumber()).to.eq(250000);
            });
        });
        it("Should get type ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.type).to.eq("pay");
            });
        });
        it("Should check if native payment ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.isNativePayment).to.eq(true);
            });
        });
        it("Should get currency name ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.currencyName).to.eq("ALGO");
            });
        });
        it("Should get elementary unit ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.elementaryUnits.toNumber()).to.eq(1000000);
            });
        });
        it("Should get success status ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.successStatus).to.eq(0);
            });
        });
        it("Should get payment summary", function () {
            var _a, _b;
            return __awaiter(this, void 0, void 0, function* () {
                const summary = yield transaction.paymentSummary(MccClient);
                (0, chai_1.expect)(summary.isNativePayment).to.eq(true);
                (0, chai_1.expect)(summary.sourceAddress).to.eq("XNFT36FUCFRR6CK675FW4BEBCCCOJ4HOSMGCN6J2W6ZMB34KM2ENTNQCP4");
                (0, chai_1.expect)(summary.receivingAddress).to.eq("5VA3MVA4SPRDSXK4X3YJJNCAO23SP4H57XLKZHRXZ3V7HRCF4BK4P66AJ4");
                (0, chai_1.expect)((_a = summary.spentAmount) === null || _a === void 0 ? void 0 : _a.toNumber()).to.eq(251000);
                (0, chai_1.expect)((_b = summary.receivedAmount) === null || _b === void 0 ? void 0 : _b.toNumber()).to.eq(250000);
                (0, chai_1.expect)(summary.paymentReference).to.eq("0x0000000000000000000000000000000000000000000000000000000000000000");
                (0, chai_1.expect)(summary.oneToOne).to.eq(true);
                (0, chai_1.expect)(summary.isFull).to.eq(true);
            });
        });
    });
});
//# sourceMappingURL=transaction.test.js.map