"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.sendUtxoTransaction = exports.sendMinimalUTXOTransaction = exports.round_for_ltc = void 0;
const index_1 = require("../../src/index");
const chai_1 = require("chai");
const reg_tests_url = process.env.BTC_URL_REGTEST || "";
const reg_test_user = process.env.BTC_USERNAME_REGTEST || "";
const reg_test_pass = process.env.BTC_PASSWORD_REGTEST || "";
function round_for_ltc(input) {
    return parseFloat(input.toFixed(8));
}
exports.round_for_ltc = round_for_ltc;
function sendMinimalUTXOTransaction(RPC, fromWalletLabel, from, to) {
    return __awaiter(this, void 0, void 0, function* () {
        let unspent = yield RPC.listUnspentTransactions(fromWalletLabel);
        let vin = [];
        let vinaddresses = [];
        let unspentAmount = 0;
        for (let utx of unspent) {
            if (utx.address == from) {
                vin.push({
                    txid: utx.txid,
                    vout: utx.vout,
                });
                vinaddresses.push(utx.address);
                unspentAmount += utx.amount;
            }
        }
        const gas = 1e-5;
        const test_amount = 1e-5;
        if (unspentAmount < gas + test_amount) {
            console.error("Not enough founds on sender wallet");
            throw Error("Insufficient founds on sender wallet");
        }
        let vout = [
            { address: to, amount: test_amount },
            { address: from, amount: round_for_ltc(unspentAmount - test_amount - gas) },
        ];
        let a = yield RPC.createRawTransaction(fromWalletLabel, vin, vout);
        let signkeys = [];
        for (let add of vinaddresses) {
            signkeys.push(yield RPC.getPrivateKey(fromWalletLabel, add));
        }
        let signedTx = yield RPC.signRawTransaction(fromWalletLabel, a, signkeys);
        const txId = yield RPC.sendRawTransactionInBlock(fromWalletLabel, signedTx.hex);
        return txId;
    });
}
exports.sendMinimalUTXOTransaction = sendMinimalUTXOTransaction;
function sendUtxoTransaction(RPC, fromWalletLabel, from, to, amount, gas = 1e-5) {
    return __awaiter(this, void 0, void 0, function* () {
        let unspent = yield RPC.listUnspentTransactions(fromWalletLabel);
        let vin = [];
        let vinaddresses = [];
        let unspentAmount = 0;
        for (let utx of unspent) {
            if (utx.address == from) {
                vin.push({
                    txid: utx.txid,
                    vout: utx.vout,
                });
                vinaddresses.push(utx.address);
                unspentAmount += utx.amount;
            }
        }
        const test_amount = amount - gas;
        if (unspentAmount < gas + test_amount) {
            console.error("Not enough founds on sender wallet");
            throw Error("Insufficient founds on sender wallet");
        }
        let vout = [
            { address: to, amount: test_amount },
            { address: from, amount: round_for_ltc(unspentAmount - test_amount - gas) },
        ];
        let a = yield RPC.createRawTransaction(fromWalletLabel, vin, vout);
        let signkeys = [];
        for (let add of vinaddresses) {
            signkeys.push(yield RPC.getPrivateKey(fromWalletLabel, add));
        }
        let signedTx = yield RPC.signRawTransaction(fromWalletLabel, a, signkeys);
        // const txId = await RPC.sendRawTransactionInBlock(fromWalletLabel, signedTx.hex);
        const txId = yield RPC.sendRawTransaction(fromWalletLabel, signedTx.hex);
        return txId;
    });
}
exports.sendUtxoTransaction = sendUtxoTransaction;
describe("BTC client tests", () => {
    describe("General functionalities", function () {
        it("should get block height from regtest network", function () {
            return __awaiter(this, void 0, void 0, function* () {
                const BtcRpc = new index_1.MCC.BTC({ url: reg_tests_url, username: reg_test_user, password: reg_test_pass });
                let a = yield BtcRpc.getBlockHeight();
                (0, chai_1.expect)(a).to.greaterThan(100);
            });
        });
    });
    it.skip("Test transactions", function () {
        return __awaiter(this, void 0, void 0, function* () {
            const nBtcRpc = new index_1.MCC.BTC({ url: reg_tests_url, username: reg_test_user, password: reg_test_pass, inRegTest: true });
            const test_wall = "test_wall_2";
            try {
                yield nBtcRpc.createWallet(test_wall);
                // load wallet
            }
            catch (_a) { }
            let new_add = yield nBtcRpc.createAddress(test_wall, "test_add", "bech32");
            let new_add_2 = yield nBtcRpc.createAddress(test_wall, "test_add2", "bech32");
            console.log("From");
            console.log(new_add);
            console.log("To:");
            console.log(new_add_2);
            let found = yield nBtcRpc.fundAddress(new_add, 10);
            console.log("fund the address");
            console.log(found);
            let bb = yield sendMinimalUTXOTransaction(nBtcRpc, test_wall, new_add, new_add_2);
            console.log("transakcija");
            console.log(bb);
            // let tx = await nBtcRpc.getTransaction(found,{verbose:true})
            // console.log(tx);
        });
    });
    it("should be able to get addresses list ", function () {
        return __awaiter(this, void 0, void 0, function* () {
            const nBtcRpc = new index_1.MCC.BTC({ url: reg_tests_url, username: reg_test_user, password: reg_test_pass });
            let addresses = yield nBtcRpc.listAllAddressesByLabel("test_wall", "test_add");
            // console.log(addresses);
            (0, chai_1.expect)(addresses.length).to.greaterThan(0);
        });
    });
    it("should get no such transaction response ", function () {
        return __awaiter(this, void 0, void 0, function* () {
            const nBtcRpc = new index_1.MCC.BTC({ url: reg_tests_url, username: reg_test_user, password: reg_test_pass });
            let transaction = yield nBtcRpc.getTransaction("ab2267bafb5a4d93486b451cf9de79dec478763e21ddd22705a24ae3aa856ec7");
            console.log(transaction);
            // expect(addresses.length).to.greaterThan(0);
        });
    });
    it("should check get transaction response typings ", function () {
        return __awaiter(this, void 0, void 0, function* () {
            const nBtcRpc = new index_1.MCC.BTC({ url: reg_tests_url, username: reg_test_user, password: reg_test_pass, inRegTest: true });
            const tx = "ab2267bafb5a4d93486b451cf9de79dec478763e21ddd22705a24ae3aa856ec3";
            let txres = yield nBtcRpc.getTransaction(tx);
            (0, chai_1.expect)(typeof txres).to.equal("string");
        });
    });
    describe("All possible transaction types", function () {
        let possibleRPC;
        let addresses_leg = [];
        let addresses_leg_2 = [];
        let addresses_seg = [];
        let addresses_seg_2 = [];
        let addresses_bec = [];
        let addresses_bec_2 = [];
        const test_wall = "all_possible_wallet";
        const test_acc_leg = "all_possible_add_leg";
        const test_acc_seg = "all_possible_add_seg";
        const test_acc_bec = "all_possible_add_bec";
        const test_acc_2_leg = "all_possible_add_leg_rec";
        const test_acc_2_seg = "all_possible_add_seg_rec";
        const test_acc_2_bec = "all_possible_add_bec_rec";
        before(function () {
            return __awaiter(this, void 0, void 0, function* () {
                possibleRPC = new index_1.MCC.BTC({ url: reg_tests_url, username: reg_test_user, password: reg_test_pass, inRegTest: true });
                try {
                    yield possibleRPC.createWallet(test_wall);
                }
                catch (_a) {
                    try {
                        yield possibleRPC.loadWallet(test_wall);
                    }
                    catch (_b) { }
                }
                try {
                    addresses_leg = yield possibleRPC.listAllAddressesByLabel(test_wall, test_acc_leg);
                }
                catch (_c) { }
                if (addresses_leg.length === 0) {
                    // we need to create legacy address
                    yield possibleRPC.createAddress(test_wall, test_acc_leg, "legacy");
                }
                try {
                    addresses_leg_2 = yield possibleRPC.listAllAddressesByLabel(test_wall, test_acc_2_leg);
                }
                catch (_d) { }
                if (addresses_leg_2.length === 0) {
                    // we need to create legacy address
                    yield possibleRPC.createAddress(test_wall, test_acc_2_leg, "legacy");
                }
                try {
                    addresses_seg = yield possibleRPC.listAllAddressesByLabel(test_wall, test_acc_seg);
                }
                catch (_e) { }
                if (addresses_seg.length === 0) {
                    // we need to create legacy address
                    yield possibleRPC.createAddress(test_wall, test_acc_seg, "p2sh-segwit");
                }
                try {
                    addresses_seg_2 = yield possibleRPC.listAllAddressesByLabel(test_wall, test_acc_2_seg);
                }
                catch (_f) { }
                if (addresses_seg_2.length === 0) {
                    // we need to create legacy address
                    yield possibleRPC.createAddress(test_wall, test_acc_2_seg, "p2sh-segwit");
                }
                try {
                    addresses_bec = yield possibleRPC.listAllAddressesByLabel(test_wall, test_acc_bec);
                }
                catch (_g) { }
                if (addresses_bec.length === 0) {
                    // we need to create legacy address
                    yield possibleRPC.createAddress(test_wall, test_acc_bec, "bech32");
                }
                try {
                    addresses_bec_2 = yield possibleRPC.listAllAddressesByLabel(test_wall, test_acc_2_bec);
                }
                catch (_h) { }
                if (addresses_bec_2.length === 0) {
                    // we need to create legacy address
                    yield possibleRPC.createAddress(test_wall, test_acc_2_bec, "bech32");
                }
                addresses_leg = yield possibleRPC.listAllAddressesByLabel(test_wall, test_acc_leg);
                addresses_leg_2 = yield possibleRPC.listAllAddressesByLabel(test_wall, test_acc_2_leg);
                addresses_seg = yield possibleRPC.listAllAddressesByLabel(test_wall, test_acc_seg);
                addresses_seg_2 = yield possibleRPC.listAllAddressesByLabel(test_wall, test_acc_2_seg);
                addresses_bec = yield possibleRPC.listAllAddressesByLabel(test_wall, test_acc_bec);
                addresses_bec_2 = yield possibleRPC.listAllAddressesByLabel(test_wall, test_acc_2_bec);
            });
        });
        it("should create tx legacy -> legacy", function () {
            return __awaiter(this, void 0, void 0, function* () {
                yield possibleRPC.fundAddress(addresses_leg[0].address, 10);
                let ttx = yield sendUtxoTransaction(possibleRPC, test_wall, addresses_leg[0].address, addresses_leg_2[0].address, 10 - 1e-4, 1e-4);
                console.log("Legacy -> Legacy: ", ttx);
            });
        });
        it("should create tx p2sh-segwit -> p2sh-segwit", function () {
            return __awaiter(this, void 0, void 0, function* () {
                yield possibleRPC.fundAddress(addresses_seg[0].address, 10);
                let ttx = yield sendUtxoTransaction(possibleRPC, test_wall, addresses_seg[0].address, addresses_seg_2[0].address, 10 - 1e-4, 1e-4);
                console.log("p2sh-segwit -> p2sh-segwit: ", ttx);
            });
        });
        it("should create tx bech32 -> bech32", function () {
            return __awaiter(this, void 0, void 0, function* () {
                yield possibleRPC.fundAddress(addresses_bec[0].address, 10);
                let ttx = yield sendUtxoTransaction(possibleRPC, test_wall, addresses_bec[0].address, addresses_bec_2[0].address, 10 - 1e-4, 1e-4);
                console.log("bech32 -> bech32: ", ttx);
            });
        });
        it("should log all addresses: ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(addresses_leg);
                console.log(addresses_leg_2);
                console.log(addresses_seg);
                console.log(addresses_seg_2);
                console.log(addresses_bec);
                console.log(addresses_bec_2);
            });
        });
    });
});
//# sourceMappingURL=btcClient.test.js.map