"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const src_1 = require("../../src");
const BtcMccConnection = {
    url: process.env.BTC_URL || "",
    username: process.env.BTC_USERNAME || "",
    password: process.env.BTC_PASSWORD || "",
};
describe("Transaction Btc test ", function () {
    let MccClient;
    before(function () {
        return __awaiter(this, void 0, void 0, function* () {
            MccClient = new src_1.MCC.BTC(BtcMccConnection);
        });
    });
    describe("Transaction full processed ", function () {
        const txid = "0x4d58bd737d87eb2011ba7e0d33d3a0c8f94f13020dd096f8d1b96849c9660a53";
        let transaction;
        before(() => __awaiter(this, void 0, void 0, function* () {
            let fullTrans = yield MccClient.getTransaction(txid);
            if (fullTrans) {
                transaction = new src_1.BtcTransaction(fullTrans.data);
                yield transaction.makeFullPayment(MccClient);
            }
        }));
        it("Should get transaction txid ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.txid).to.eq("4d58bd737d87eb2011ba7e0d33d3a0c8f94f13020dd096f8d1b96849c9660a53");
            });
        });
        it("Should get standardized txid ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.stdTxid).to.eq("4d58bd737d87eb2011ba7e0d33d3a0c8f94f13020dd096f8d1b96849c9660a53");
            });
        });
        it("Should get transaction hash ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.hash).to.eq("c16f1d89e5e82ecfebe57f3ff5fc1c7a6144964a133da24ec3216965cf787cfd");
            });
        });
        it("Should get transaction reference array ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.reference.length).to.eq(0);
            });
        });
        it("Should get standardized transaction reference ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.stdPaymentReference).to.eq("0x0000000000000000000000000000000000000000000000000000000000000000");
            });
        });
        it("Should get transaction timestamp ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                // From https://blockstream.info/tx/4d58bd737d87eb2011ba7e0d33d3a0c8f94f13020dd096f8d1b96849c9660a53
                (0, chai_1.expect)(transaction.unixTimestamp).to.eq(1650342330);
            });
        });
        it("Should get source address ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.sourceAddresses.length).to.eq(2);
                (0, chai_1.expect)(transaction.sourceAddresses[0]).to.eq("bc1qdl753ur9ucwa3cgfrud2nqvu7k69dykk3cwwx6g64a5szn3xw92sp8mc7a");
                (0, chai_1.expect)(transaction.sourceAddresses[1]).to.eq("bc1qdl753ur9ucwa3cgfrud2nqvu7k69dykk3cwwx6g64a5szn3xw92sp8mc7a");
            });
        });
        it("Should get receiving address ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.receivingAddresses.length).to.eq(2);
                (0, chai_1.expect)(transaction.receivingAddresses[0]).to.eq("1KiJkugknjgW6AHXNgVQgNuo3b5DqsVFmk");
                (0, chai_1.expect)(transaction.receivingAddresses[1]).to.eq("bc1qwqdg6squsna38e46795at95yu9atm8azzmyvckulcc7kytlcckxswvvzej");
            });
        });
        it("Should get fee ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.fee.toNumber()).to.eq(40000);
            });
        });
        it("Should spend amount ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.spentAmounts.length).to.eq(2);
                (0, chai_1.expect)(transaction.spentAmounts[0].address).to.eq("bc1qdl753ur9ucwa3cgfrud2nqvu7k69dykk3cwwx6g64a5szn3xw92sp8mc7a");
                (0, chai_1.expect)(transaction.spentAmounts[0].amount.toNumber()).to.eq(400000000);
                (0, chai_1.expect)(transaction.spentAmounts[1].address).to.eq("bc1qdl753ur9ucwa3cgfrud2nqvu7k69dykk3cwwx6g64a5szn3xw92sp8mc7a");
                (0, chai_1.expect)(transaction.spentAmounts[1].amount.toNumber()).to.eq(400000000);
            });
        });
        it("Should received amount ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.receivedAmounts.length).to.eq(2);
                (0, chai_1.expect)(transaction.receivedAmounts[0].address).to.eq("1KiJkugknjgW6AHXNgVQgNuo3b5DqsVFmk");
                (0, chai_1.expect)(transaction.receivedAmounts[0].amount.toNumber()).to.eq(494000000);
                (0, chai_1.expect)(transaction.receivedAmounts[1].address).to.eq("bc1qwqdg6squsna38e46795at95yu9atm8azzmyvckulcc7kytlcckxswvvzej");
                (0, chai_1.expect)(transaction.receivedAmounts[1].amount.toNumber()).to.eq(305960000);
            });
        });
        it("Should get type ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.type).to.eq("full_payment");
            });
        });
        it("Should check if native payment ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.isNativePayment).to.eq(true);
            });
        });
        it("Should get currency name ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.currencyName).to.eq("BTC");
            });
        });
        it("Should get elementary unit ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.elementaryUnits.toNumber()).to.eq(100000000);
            });
        });
        it("Should get success status ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                (0, chai_1.expect)(transaction.successStatus).to.eq(0);
            });
        });
        it("Should get payment summary from utxo 0 to utxo 0", function () {
            var _a, _b;
            return __awaiter(this, void 0, void 0, function* () {
                const summary = yield transaction.paymentSummary(MccClient, 0, 0, false);
                (0, chai_1.expect)(summary.isNativePayment).to.eq(true);
                (0, chai_1.expect)(summary.sourceAddress).to.eq("bc1qdl753ur9ucwa3cgfrud2nqvu7k69dykk3cwwx6g64a5szn3xw92sp8mc7a");
                (0, chai_1.expect)(summary.receivingAddress).to.eq("1KiJkugknjgW6AHXNgVQgNuo3b5DqsVFmk");
                (0, chai_1.expect)((_a = summary.spentAmount) === null || _a === void 0 ? void 0 : _a.toNumber()).to.eq(800000000);
                (0, chai_1.expect)((_b = summary.receivedAmount) === null || _b === void 0 ? void 0 : _b.toNumber()).to.eq(494000000);
                (0, chai_1.expect)(summary.paymentReference).to.eq("0x0000000000000000000000000000000000000000000000000000000000000000");
                (0, chai_1.expect)(summary.oneToOne).to.eq(false);
                (0, chai_1.expect)(summary.isFull).to.eq(true);
            });
        });
        it("Should get payment summary from utxo 0 to utxo 1", function () {
            var _a, _b;
            return __awaiter(this, void 0, void 0, function* () {
                const summary = yield transaction.paymentSummary(MccClient, 0, 1, false);
                (0, chai_1.expect)(summary.isNativePayment).to.eq(true);
                (0, chai_1.expect)(summary.sourceAddress).to.eq("bc1qdl753ur9ucwa3cgfrud2nqvu7k69dykk3cwwx6g64a5szn3xw92sp8mc7a");
                (0, chai_1.expect)(summary.receivingAddress).to.eq("bc1qwqdg6squsna38e46795at95yu9atm8azzmyvckulcc7kytlcckxswvvzej");
                (0, chai_1.expect)((_a = summary.spentAmount) === null || _a === void 0 ? void 0 : _a.toNumber()).to.eq(800000000);
                (0, chai_1.expect)((_b = summary.receivedAmount) === null || _b === void 0 ? void 0 : _b.toNumber()).to.eq(305960000);
                (0, chai_1.expect)(summary.paymentReference).to.eq("0x0000000000000000000000000000000000000000000000000000000000000000");
                (0, chai_1.expect)(summary.oneToOne).to.eq(false);
                (0, chai_1.expect)(summary.isFull).to.eq(true);
            });
        });
        it("Should get payment summary from utxo 1 to utxo 0", function () {
            var _a, _b;
            return __awaiter(this, void 0, void 0, function* () {
                const summary = yield transaction.paymentSummary(MccClient, 1, 0, false);
                (0, chai_1.expect)(summary.isNativePayment).to.eq(true);
                (0, chai_1.expect)(summary.sourceAddress).to.eq("bc1qdl753ur9ucwa3cgfrud2nqvu7k69dykk3cwwx6g64a5szn3xw92sp8mc7a");
                (0, chai_1.expect)(summary.receivingAddress).to.eq("1KiJkugknjgW6AHXNgVQgNuo3b5DqsVFmk");
                (0, chai_1.expect)((_a = summary.spentAmount) === null || _a === void 0 ? void 0 : _a.toNumber()).to.eq(800000000);
                (0, chai_1.expect)((_b = summary.receivedAmount) === null || _b === void 0 ? void 0 : _b.toNumber()).to.eq(494000000);
                (0, chai_1.expect)(summary.paymentReference).to.eq("0x0000000000000000000000000000000000000000000000000000000000000000");
                (0, chai_1.expect)(summary.oneToOne).to.eq(false);
                (0, chai_1.expect)(summary.isFull).to.eq(true);
            });
        });
        it("Should get payment summary from utxo 1 to utxo 1", function () {
            var _a, _b;
            return __awaiter(this, void 0, void 0, function* () {
                const summary = yield transaction.paymentSummary(MccClient, 1, 1, false);
                (0, chai_1.expect)(summary.isNativePayment).to.eq(true);
                (0, chai_1.expect)(summary.sourceAddress).to.eq("bc1qdl753ur9ucwa3cgfrud2nqvu7k69dykk3cwwx6g64a5szn3xw92sp8mc7a");
                (0, chai_1.expect)(summary.receivingAddress).to.eq("bc1qwqdg6squsna38e46795at95yu9atm8azzmyvckulcc7kytlcckxswvvzej");
                (0, chai_1.expect)((_a = summary.spentAmount) === null || _a === void 0 ? void 0 : _a.toNumber()).to.eq(800000000);
                (0, chai_1.expect)((_b = summary.receivedAmount) === null || _b === void 0 ? void 0 : _b.toNumber()).to.eq(305960000);
                (0, chai_1.expect)(summary.paymentReference).to.eq("0x0000000000000000000000000000000000000000000000000000000000000000");
                (0, chai_1.expect)(summary.oneToOne).to.eq(false);
                (0, chai_1.expect)(summary.isFull).to.eq(true);
            });
        });
    });
});
//# sourceMappingURL=transactionObject.test.js.map