"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const src_1 = require("../../src");
const DogeMccConnection = {
    url: process.env.DOGE_URL || "",
    username: process.env.DOGE_USERNAME || "",
    password: process.env.DOGE_PASSWORD || "",
};
describe("DOGE mainnet client tests", () => {
    describe("get transaction tests", function () {
        it("should get transaction ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                const RPC = new src_1.MCC.DOGE(DogeMccConnection);
                const txid = "2d906dbce50eb47567d1decae6a0ce5267eaabe56838ea9fd700a732bbcdcb3b";
                let trans = yield RPC.getTransaction(txid);
                if (trans) {
                    (0, chai_1.expect)(trans.hash).to.eq(txid);
                }
                // console.log(trans);
            });
        });
        it("should return null if transaction does not exist ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                const RPC = new src_1.MCC.DOGE(DogeMccConnection);
                const txid = "2d906dbce50eb47567d1decae6a0ce5267eaabe56838ea9fd700a732bbcdcbff";
                let trans = yield RPC.getTransaction(txid);
                if (trans) {
                    (0, chai_1.expect)(trans).to.equal(null);
                }
                // await expectThrow(trans,{ code: -5, message: 'Block not found' })
            });
        });
        it("should be able to get block height ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                const RPC = new src_1.MCC.DOGE(DogeMccConnection);
                let blockHeight = yield RPC.getBlockHeight();
                (0, chai_1.expect)(blockHeight).to.be.above(0);
            });
        });
        it("should be able to get block height ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                const RPC = new src_1.MCC.DOGE(DogeMccConnection);
                let blockHeight = yield RPC.getBlockHeight();
                (0, chai_1.expect)(blockHeight).to.be.above(0);
            });
        });
        it("should be able to get block header from height ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                const RPC = new src_1.MCC.DOGE(DogeMccConnection);
                const height = 4103881;
                let blockHeader = yield RPC.getBlockHeader(height);
                (0, chai_1.expect)(blockHeader).to.not.eq(null);
            });
        });
        it("should be able to get block header from hash ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                const RPC = new src_1.MCC.DOGE(DogeMccConnection);
                const hash = "8ea32399901c056e32c0fef84208bc257824f6718a6d4906ab00833b8d87ea89";
                let blockHeader = yield RPC.getBlockHeader(hash);
                (0, chai_1.expect)(blockHeader).to.not.eq(null);
            });
        });
        it("should be able to get block from height ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                const RPC = new src_1.MCC.DOGE(DogeMccConnection);
                const height = 4103881;
                let blockHeader = yield RPC.getBlock(height);
                (0, chai_1.expect)(blockHeader).to.not.eq(null);
            });
        });
        it("should be able to get block from hash ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                const RPC = new src_1.MCC.DOGE(DogeMccConnection);
                const hash = "8ea32399901c056e32c0fef84208bc257824f6718a6d4906ab00833b8d87ea89";
                let blockHeader = yield RPC.getBlock(hash);
                (0, chai_1.expect)(blockHeader).to.not.eq(null);
            });
        });
        it("should get full transaction ", function () {
            return __awaiter(this, void 0, void 0, function* () {
                const RPC = new src_1.MCC.DOGE(DogeMccConnection);
                const txid = "2d906dbce50eb47567d1decae6a0ce5267eaabe56838ea9fd700a732bbcdcb3b";
                let trans = yield RPC.getTransaction(txid);
                // console.log(trans);
                if (trans) {
                    (0, chai_1.expect)(trans.hash).to.eq(txid);
                }
            });
        });
    });
    describe("ChainTips", function () {
        return __awaiter(this, void 0, void 0, function* () {
            it("basic chaintips ", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const BtcRpc = new src_1.MCC.BTC(DogeMccConnection);
                    const chaintips = yield BtcRpc.getTopBlocks();
                    // console.log(chaintips);
                    console.log(chaintips.length);
                });
            });
            it("full chaintips ", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const BtcRpc = new src_1.MCC.BTC(DogeMccConnection);
                    const chaintips = yield BtcRpc.getTopBlocks({ all_blocks: true });
                    // console.log(chaintips);
                    console.log(chaintips.length);
                });
            });
            it("chaintips after ", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const BtcRpc = new src_1.MCC.BTC(DogeMccConnection);
                    const chaintips = yield BtcRpc.getTopBlocks({ height_gte: 4133821 });
                    console.log(chaintips);
                    console.log(chaintips.length);
                });
            });
        });
    });
});
//# sourceMappingURL=mainNetDoge.test.js.map