"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const src_1 = require("../../src");
const xrpUtils_1 = require("../../src/utils/xrpUtils");
const XrpAddress = require("ripple-address-codec");
const XRPMccConnection = {
    url: process.env.XRP_URL || "",
    username: process.env.XRP_USERNAME || "",
    password: process.env.XRP_PASSWORD || "",
};
describe("Xrpl account test mainnet ", function () {
    let MccClient;
    before(function () {
        return __awaiter(this, void 0, void 0, function* () {
            MccClient = new src_1.MCC.XRP(XRPMccConnection);
        });
    });
    describe("account info", () => __awaiter(this, void 0, void 0, function* () {
        it(`Should get account info`, () => __awaiter(this, void 0, void 0, function* () {
            const acc = "rG1QQv2nh2gr7RCZ1P8YYcBUKCCN633jCn";
            const info = yield MccClient.getAccountInfo(acc);
            console.log(info);
            // Get the flags
            const flags = (0, xrpUtils_1.processFlags)(info.result.account_data.Flags);
            console.log(flags);
        }));
        it(`Should get account info 2 `, () => __awaiter(this, void 0, void 0, function* () {
            const acc = "rMv1dFonUTJo4qeQDmmCuz3cXPsxy9AiHz";
            const info = yield MccClient.getAccountInfo(acc);
            console.log(info);
            // Get the flags
            const flags = (0, xrpUtils_1.processFlags)(info.result.account_data.Flags);
            console.log(flags);
            // get account founding transaction (account create)
            const fTx = "5439BB6872644CE74A09F4654CECFC5F443E9687AD5EF6D119E729898F41A8C5";
            const trans = yield MccClient.getTransaction(fTx);
            if (trans) {
                console.log(trans.data);
                console.log(yield trans.paymentSummary(MccClient));
                console.log(trans.isAccountCreate);
            }
        }));
    }));
    describe("account transactions", () => __awaiter(this, void 0, void 0, function* () {
        it(`Should get account transactions`, () => __awaiter(this, void 0, void 0, function* () {
            // const acc = 'rEVd9QP2aGsJ7wuty2qV8gqMiTiA1sX2j1'
            const acc = "r4BhzWSGGjTeSdpcXMPoT1AbiCQm76FQGd";
            console.log("Decoded address", acc);
            const byts = XrpAddress.decodeAccountID(acc); // TODO add to utils
            console.log(byts, byts.length);
            const encb = XrpAddress.encodeAccountID(byts);
            console.log("Back enc: ", encb);
            console.log("Original: ", acc);
            // get account transactions
            const tr = yield MccClient.getAccountTransactions(acc, 71859000, 71867500);
            // const tr = await MccClient.getAccountTransactions(acc)
            console.log(tr);
            console.log(tr.result.transactions.length);
        }));
    }));
});
const XRPMccConnectionTest = {
    url: process.env.XRP_URL_TESTNET || "",
    username: process.env.XRP_USERNAME_TESTNET || "",
    password: process.env.XRP_PASSWORD_TESTNET || "",
};
describe("Xrpl account test testnet ", function () {
    let MccClient;
    before(function () {
        return __awaiter(this, void 0, void 0, function* () {
            MccClient = new src_1.MCC.XRP(XRPMccConnectionTest);
        });
    });
    describe("account info", () => __awaiter(this, void 0, void 0, function* () {
        it(`Should get account info`, () => __awaiter(this, void 0, void 0, function* () {
            var _a, _b, _c;
            const info = yield MccClient.getAccountInfo("rBwD7GqAPFoZvzz6YaR5HyJWD8TUoaUbJo");
            console.log(info);
            // Get the flags
            console.time("Flag new1");
            const flags = (0, xrpUtils_1.processFlags)(info.result.account_data.Flags);
            console.timeEnd("Flag new1");
            console.time("Flag old1");
            const oldFlags = (0, xrpUtils_1.processFlagsOld1)(info.result.account_data.Flags);
            console.timeEnd("Flag old1");
            console.time("Flag old2");
            const fastFlags = (0, xrpUtils_1.processFlagsOld2)(info.result.account_data.Flags);
            console.timeEnd("Flag old2");
            console.log("Flags");
            console.log(flags);
            console.log(oldFlags);
            console.log(fastFlags);
            // Regular key
            console.log("Regular key");
            console.log(info.result.account_data.RegularKey);
            console.log(info.result.account_data.RegularKey === src_1.SpecialAddresses.ACCOUNT_ONE);
            // Multisig check
            console.log("Multisig");
            // TODO ripple has a problem here?
            // @ts-ignore
            console.log((_a = info.result.account_data) === null || _a === void 0 ? void 0 : _a.signer_lists);
            // @ts-ignore
            if ((_b = info.result.account_data) === null || _b === void 0 ? void 0 : _b.signer_lists) {
                // @ts-ignore
                console.log(((_c = info.result.account_data) === null || _c === void 0 ? void 0 : _c.signer_lists.length) === 0);
            }
        }));
    }));
    describe("step by step account info", () => __awaiter(this, void 0, void 0, function* () {
        const acc = "rD8btyHW512KmJdsQEoD9KFTMxfDbpqkkA"; // my acc
        it(`Should get account info at the begging`, () => __awaiter(this, void 0, void 0, function* () {
            const info = yield MccClient.getAccountInfo(acc, 28014551);
            const flags = (0, xrpUtils_1.processFlags)(info.result.account_data.Flags);
            (0, chai_1.expect)(flags.length).to.eq(0);
            (0, chai_1.expect)(info.result.account_data.RegularKey).to.not.eq(src_1.SpecialAddresses.ACCOUNT_ONE);
        }));
        // set by tx 996CF647C655904730759CE3919E5B4F4AB6E99046EB53DA0457F1956A52B6E3
        it(`Should get account info after setting default rippling`, () => __awaiter(this, void 0, void 0, function* () {
            const info = yield MccClient.getAccountInfo(acc, 28014612);
            const flags = (0, xrpUtils_1.processFlags)(info.result.account_data.Flags);
            console.log(info);
            (0, chai_1.expect)(flags.length).to.eq(1);
            (0, chai_1.expect)(flags.includes("lsfDefaultRipple")).to.eq(true);
            (0, chai_1.expect)(info.result.account_data.RegularKey).to.not.eq(src_1.SpecialAddresses.ACCOUNT_ONE);
        }));
        // set by E547D21C4F24FC313F492828730C17B442C37038393705A18C64D400D186319D
        it(`Should get account info after setting regular key to ACCOUNT_ONE`, () => __awaiter(this, void 0, void 0, function* () {
            const info = yield MccClient.getAccountInfo(acc, 28014704);
            const flags = (0, xrpUtils_1.processFlags)(info.result.account_data.Flags);
            (0, chai_1.expect)(flags.length).to.eq(2);
            (0, chai_1.expect)(flags.includes("lsfDefaultRipple")).to.eq(true);
            (0, chai_1.expect)(info.result.account_data.RegularKey).to.eq(src_1.SpecialAddresses.ACCOUNT_ONE);
        }));
        // set by CA70256B2BC1886C5B6B40DB74C3B6B385013EF6F1C66BB940B8F31CF50D1922
        it(`Should get account info after disabling master key`, () => __awaiter(this, void 0, void 0, function* () {
            const info = yield MccClient.getAccountInfo(acc, 28014733);
            const flags = (0, xrpUtils_1.processFlags)(info.result.account_data.Flags);
            console.log(info);
            console.log(flags);
            (0, chai_1.expect)(flags.length).to.eq(3);
            (0, chai_1.expect)(flags.includes("lsfDefaultRipple")).to.eq(true);
            (0, chai_1.expect)(flags.includes("lsfDisableMaster")).to.eq(true);
            (0, chai_1.expect)(info.result.account_data.RegularKey).to.eq(src_1.SpecialAddresses.ACCOUNT_ONE);
        }));
    }));
});
// testnet account
// The one rBwD7GqAPFoZvzz6YaR5HyJWD8TUoaUbJo
// my rD8btyHW512KmJdsQEoD9KFTMxfDbpqkkA
//# sourceMappingURL=accountInfo.test.js.map