"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const src_1 = require("../../src");
const XRPMccConnection = {
    url: process.env.XRP_URL || "",
    username: process.env.XRP_USERNAME || "",
    password: process.env.XRP_PASSWORD || "",
};
describe("XRP testnet client tests", () => {
    let client;
    before(function () {
        client = new src_1.MCC.XRP(XRPMccConnection);
    });
    describe("Should be able to get block height", () => __awaiter(void 0, void 0, void 0, function* () {
        it(`Should be able to get block height `, () => __awaiter(void 0, void 0, void 0, function* () {
            let height = yield client.getBlockHeight();
            console.log(height);
            (0, chai_1.expect)(height).to.be.greaterThan(70000000);
        }));
    }));
    describe("Basic functionalities", function () {
        it("Should return block if exists", () => __awaiter(this, void 0, void 0, function* () {
            let n = 69453782;
            let block = yield client.getBlock(n);
            if (block) {
                (0, chai_1.expect)(block.number).to.equal(n);
            }
            // assert(block.result.ledger_index === n);
        }));
        it("Should return null if block does not exist", () => __awaiter(this, void 0, void 0, function* () {
            let n = 694537820;
            let block = yield client.getBlock(n);
            (0, chai_1.expect)(block).to.equal(null);
            // assert(block == null);
            n *= 100;
            block = yield client.getBlock(n);
            (0, chai_1.expect)(block).to.equal(null);
            // assert(block == null);
        }));
        it("Should return transaction if exists", () => __awaiter(this, void 0, void 0, function* () {
            let txResponse = yield client.getTransaction("0569969AFDAF91BFCFF709D49FE23DD5656335AFD0A3879C03C8EFADEF83A0C2");
            (0, chai_1.expect)(txResponse).to.not.equal(null);
            // assert(txResponse?.result?.Account)
        }));
        it("Should return null if transaction does not exist", () => __awaiter(this, void 0, void 0, function* () {
            let txResponse = yield client.getTransaction("0669969AFDAF91BFCFF709D49FE23DD5656335AFD0A3879C03C8EFADEF83A0C2");
            (0, chai_1.expect)(txResponse).to.equal(null);
            // assert(txResponse == null);
        }));
    });
});
//# sourceMappingURL=testnetXrp.test.js.map