"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const src_1 = require("../../src");
const XRPMccConnection = {
    url: process.env.XRP_URL || "",
    username: process.env.XRP_USERNAME || "",
    password: process.env.XRP_PASSWORD || "",
};
describe("Transaction Xrp tests ", function () {
    let MccClient;
    before(function () {
        return __awaiter(this, void 0, void 0, function* () {
            MccClient = new src_1.MCC.XRP(XRPMccConnection);
        });
    });
    const TransactionsToTest = [
        {
            description: "Token CORE",
            txid: "546433CEADAEEDB0DF3B67221F9FE6E2041482775C4BD67F34A41476F766B189",
        },
        {
            description: "Token LOVE",
            txid: "D0DBB956BD1DEFAB0CB46246F111B567BBD2D0CA090C26AFD57782C7C8C4BCE1",
        },
        {
            description: "Token CX1",
            txid: "69BE6AAD4A8D0991E72AA23ECB15C7847A0D3D44E3B23CE6BDA53701B7E0F9D2",
        },
        {
            description: "Token Blessed",
            txid: "84464F5001B9E7FD79C448B9C5F01085ACE56E94A1F6E2A737FDE9A993086F16",
        },
    ];
    for (let toTest of TransactionsToTest) {
        describe(toTest.description, function () {
            let transaction;
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    let transactionb = yield MccClient.getTransaction(toTest.txid);
                    if (transactionb !== null) {
                        transaction = transactionb;
                    }
                    // console.log(transaction);
                });
            });
            it("Should get transaction hash ", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    console.log(transaction.hash);
                });
            });
            it("Should get transaction reference array ", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    console.log(transaction.reference);
                });
            });
            it("Should get transaction timestamp ", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    console.log(transaction.unixTimestamp);
                });
            });
            it("Should get source address ", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    console.log(transaction.sourceAddresses);
                });
            });
            it("Should get receiving address ", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    console.log(transaction.receivingAddresses);
                });
            });
            it("Should get fee ", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    console.log(transaction.fee.toString(10));
                });
            });
            it("Should received amount ", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    console.log(transaction.receivedAmounts);
                });
            });
            it("Should spend amount ", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    console.log(transaction.spentAmounts);
                });
            });
            it("Should get type ", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    console.log(transaction.type);
                });
            });
            it("Should get elementary unit ", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    console.log(transaction.elementaryUnits.toString(10));
                });
            });
            it("Should get success status ", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    console.log(transaction.successStatus);
                });
            });
            it("Should get currency name ", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    console.log(transaction.currencyName);
                });
            });
            it("Should check if native payment ", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    console.log(transaction.isNativePayment);
                });
            });
            it("Should get success Status ", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    console.log(transaction.successStatus);
                });
            });
            it("Should get payment summary ", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const summary = yield transaction.paymentSummary(MccClient);
                    console.log(summary);
                    if (summary.tokenElementaryUnits)
                        console.log("tokenElementaryUnits: ", summary.tokenElementaryUnits.toString(10));
                    if (summary.receivedTokenAmount)
                        console.log("receivedTokenAmount: ", summary.receivedTokenAmount.toString(10));
                    if (summary.spentAmount)
                        console.log("spentAmount: ", summary.spentAmount.toString(10));
                    if (summary.receivedAmount)
                        console.log("receivedAmount: ", summary.receivedAmount.toString(10));
                });
            });
            it("Should get transaction data ", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    console.log(transaction.data);
                });
            });
        });
    }
});
//# sourceMappingURL=tokenTransaction.test.js.map