"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const trace_1 = require("../src/utils/trace");
const managedTest_1 = require("./managedTest");
const chai = require('chai');
const expect = chai.expect;
chai.use(require('chai-as-promised'));
describe("Managed test", () => {
    let dec;
    before(function () {
        return __awaiter(this, void 0, void 0, function* () {
            trace_1.traceManager.displayTrace = true;
            dec = new managedTest_1.ManagedTest();
        });
    });
    beforeEach(function () {
        return __awaiter(this, void 0, void 0, function* () {
            trace_1.traceManager.clearTrace();
        });
    });
    afterEach(function () {
        return __awaiter(this, void 0, void 0, function* () {
            //traceManager.showTrace();
        });
    });
    after(function () {
        return __awaiter(this, void 0, void 0, function* () {
            trace_1.traceManager.displayTrace = false;
        });
    });
    it("Managed method sync", () => __awaiter(void 0, void 0, void 0, function* () {
        dec.method(5, 6);
        expect(trace_1.traceManager.firstTrace).to.eq(`ManagedTest.method(5,6)=36`);
    }));
    it("Managed method async", () => __awaiter(void 0, void 0, void 0, function* () {
        yield dec.asyncMethod(7, 8);
        expect(trace_1.traceManager.firstTrace).to.eq(`ManagedTest.asyncMethod(7,8)=224`);
    }));
    it("Managed getter", () => __awaiter(void 0, void 0, void 0, function* () {
        dec.getter;
        expect(trace_1.traceManager.firstTrace).to.eq(`ManagedTest.getter()=1`);
    }));
    it("Managed setter", () => __awaiter(void 0, void 0, void 0, function* () {
        dec.setter = 15;
        expect(trace_1.traceManager.firstTrace).to.eq(`ManagedTest.setter(15)`);
    }));
    it("Managed throw sync", () => __awaiter(void 0, void 0, void 0, function* () {
        expect(function () { dec.methodThrow(5, 0); }).to.throw("OutsideError");
        expect(trace_1.traceManager.firstTrace).to.eq(`ManagedTest.methodThrow(5,0)`);
    }));
    it("Managed throw sync mcc error", () => __awaiter(void 0, void 0, void 0, function* () {
        expect(function () { dec.methodThrowMccError(); }).to.throw("InvalidData");
        expect(trace_1.traceManager.firstTrace).to.eq(`ManagedTest.methodThrowMccError()`);
    }));
    it("Managed throw async", () => __awaiter(void 0, void 0, void 0, function* () {
        yield expect(dec.asyncMethodThrow(7, 0)).to.be.rejectedWith("OutsideError");
        expect(trace_1.traceManager.firstTrace).to.eq(`ManagedTest.asyncMethodThrow(7,0)`);
    }));
    it("Managed throw getter", () => __awaiter(void 0, void 0, void 0, function* () {
        expect(function () {
            dec.getterThrow;
        }).to.throw("OutsideError");
        expect(trace_1.traceManager.firstTrace).to.eq(`ManagedTest.getterThrow()`);
    }));
    it("Managed throw setter", () => __awaiter(void 0, void 0, void 0, function* () {
        expect(function () { dec.setterThrow = 15; }).to.throw("OutsideError");
        expect(trace_1.traceManager.firstTrace).to.eq(`ManagedTest.setterThrow(15)`);
    }));
    it.only("Managed nested test", () => __awaiter(void 0, void 0, void 0, function* () {
        //traceManager.displayTrace=true;
        dec.f1("123");
        trace_1.traceManager.showTrace(true, false, true);
        trace_1.traceManager.showMethods();
        expect(trace_1.traceManager.firstTrace).to.eq(`ManagedTest.f1(123)`);
    }));
});
//# sourceMappingURL=managedTest.test.js.map