export declare enum TraceMethodType {
    Descriptor = 0,
    Getter = 1,
    Function = 2
}
export declare class TraceMethod {
    className: string;
    methodName: string;
    methodType: TraceMethodType;
    source: string;
    calls: number;
    allInclusiveTime: number;
    constructor(className: string, methodName: string, methodType: TraceMethodType);
}
export declare class TraceStack {
    method: TraceMethod;
    args: any[];
    ret: any;
    callIndex: number;
    completed: boolean;
    startTime: number;
    endTime: number;
    error: Error | undefined;
    stack: string;
    level: number;
    constructor(method: TraceMethod, args: any[]);
    start(): void;
    complete(ret: any): void;
    completeException(error: Error): void;
    toString(): string;
}
export declare class TraceManager {
    displayTrace: boolean;
    methods: TraceMethod[];
    stack: TraceStack[];
    trace: TraceStack[];
    constructor();
    createMethod(className: string, methodName: string, methodType: TraceMethodType): TraceMethod;
    start(className: string, methodName: string, args: any[], methodType: TraceMethodType): TraceStack;
    complete(ret: any): void;
    completeError(error: Error): void;
    displayError(error: any): void;
    showStack(): void;
    showTrace(): void;
}
export declare const traceManager: TraceManager;
export declare function isPromise(p: any): boolean;
export declare function Trace(): (target: any, name?: string, descriptor?: any) => any;
//# sourceMappingURL=trace.d.ts.map