"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Trace = exports.isPromise = exports.traceManager = exports.TraceManager = exports.TraceStack = exports.TraceMethod = exports.TraceMethodType = void 0;
const errors_1 = require("./errors");
var TraceMethodType;
(function (TraceMethodType) {
    TraceMethodType[TraceMethodType["Descriptor"] = 0] = "Descriptor";
    TraceMethodType[TraceMethodType["Getter"] = 1] = "Getter";
    TraceMethodType[TraceMethodType["Function"] = 2] = "Function";
})(TraceMethodType = exports.TraceMethodType || (exports.TraceMethodType = {}));
class TraceMethod {
    constructor(className, methodName, methodType) {
        this.className = "";
        this.methodName = "";
        this.source = "";
        this.calls = 0;
        this.allInclusiveTime = 0;
        this.className = className;
        this.methodName = methodName;
        this.calls = 0;
        this.methodType = methodType;
    }
}
exports.TraceMethod = TraceMethod;
class TraceStack {
    constructor(method, args) {
        this.ret = null;
        // timing
        this.completed = false;
        this.startTime = 0;
        this.endTime = 0;
        // stack
        this.stack = "";
        this.level = 0;
        this.method = method;
        this.callIndex = method.calls;
        this.method.calls++;
        this.args = args;
    }
    start() {
        // todo: timing
        this.startTime = 0;
        this.stack = Error().stack;
        const trace = this.stack.split("\n");
        if (trace.length > 4) {
            const traceLine = /[^(]*\(([^)]*)\)/.exec(trace[4]);
            if (traceLine && traceLine.length > 1) {
                this.method.source = traceLine[1];
            }
        }
    }
    complete(ret) {
        // todo: timing
        this.endTime = 0;
        this.method.allInclusiveTime += this.endTime - this.startTime;
        this.completed = true;
        this.ret = ret;
    }
    completeException(error) {
        this.complete(undefined);
        this.error = error;
    }
    toString() {
        // todo: args are not properly displayed
        // todo: not completed
        // todo: error
        if (this.ret) {
            return `${this.method.className}.${this.method.methodName}(${this.args.map((val) => "%s").join(", ")})=${this.ret} [${this.method.source}]`;
        }
        else {
            return `${this.method.className}.${this.method.methodName}(${this.args.map((val) => "%s").join(", ")}) [${this.method.source}]`;
        }
    }
}
exports.TraceStack = TraceStack;
class TraceManager {
    constructor() {
        this.displayTrace = false;
        this.methods = [];
        this.stack = [];
        this.trace = [];
    }
    createMethod(className, methodName, methodType) {
        let method = this.methods.find(x => x.className == className && x.methodName == methodName);
        if (method)
            return method;
        method = new TraceMethod(className, methodName, methodType);
        this.methods.push(method);
        return method;
    }
    start(className, methodName, args, methodType) {
        const method = this.createMethod(className, methodName, methodType);
        const trace = new TraceStack(method, args);
        trace.start();
        trace.level = this.stack.length;
        this.stack.push(trace);
        this.trace.push(trace);
        if (this.displayTrace) {
            console.log(`trace: ${trace.toString()}`);
        }
        return trace;
    }
    complete(ret) {
        var _a;
        (_a = this.stack.pop()) === null || _a === void 0 ? void 0 : _a.complete(ret);
    }
    completeError(error) {
        var _a;
        (_a = this.stack.pop()) === null || _a === void 0 ? void 0 : _a.completeException(error);
    }
    displayError(error) {
        console.log("STACK");
        this.showStack();
    }
    showStack() {
        for (let trace of this.stack) {
            console.log(trace.toString());
        }
    }
    showTrace() {
        for (let trace of this.trace) {
            console.log(trace.toString());
        }
    }
}
exports.TraceManager = TraceManager;
exports.traceManager = new TraceManager();
function TraceStub(target, name, funct, cx, args, methodType) {
    exports.traceManager.start(target.constructor.name, name, ["get"], methodType);
    // try {
    //    let ret = cx ? funct.apply( cx , args ) : funct(); 
    //    if (typeof ret.then === "function") {
    //       ret.then(() => traceManager.complete(ret));
    //    } else {
    //       traceManager.complete(ret);
    //    }
    //    return ret;
    // }
    // catch (error) {
    //    traceManager.catch(error);
    // }
    try {
        let res = cx ? funct.apply(cx, args) : funct();
        if (!isPromise(res)) {
            exports.traceManager.complete(res);
            return res;
        }
        return new Promise((resolve, reject) => {
            res.then((result) => {
                exports.traceManager.complete(result);
                resolve(result);
            }).catch((error) => {
                if ((error === null || error === void 0 ? void 0 : error.name) === errors_1.MCC_ERROR) {
                    exports.traceManager.completeError(error);
                    exports.traceManager.displayError(error);
                    reject(error);
                }
                else {
                    const newError = new errors_1.mccOutsideError(error);
                    exports.traceManager.completeError(newError);
                    exports.traceManager.displayError(newError);
                    reject(newError);
                }
            });
        });
    }
    catch (error) {
        if ((error === null || error === void 0 ? void 0 : error.name) === errors_1.MCC_ERROR) {
            exports.traceManager.completeError(error);
            exports.traceManager.displayError(error);
            throw error;
        }
        else {
            const newError = new errors_1.mccOutsideError(error);
            exports.traceManager.completeError(newError);
            exports.traceManager.displayError(newError);
            throw new errors_1.mccOutsideError(newError);
        }
    }
}
function RegisterTraceValue(target, name, descriptor) {
    //  decorating a method
    if (descriptor && descriptor.value) {
        let original = descriptor.value;
        descriptor.value = function (...args) {
            TraceStub(target.constructor.name, name, original, this, args, TraceMethodType.Descriptor);
        };
        return descriptor;
    }
    return null;
}
function isPromise(p) {
    if (typeof p === 'object' && typeof p.then === 'function') {
        return true;
    }
    return false;
}
exports.isPromise = isPromise;
function RegisterTraceGetter(target, name, descriptor) {
    if (descriptor && descriptor.get) {
        let original = descriptor.get;
        descriptor.get = function () {
            TraceStub(target.constructor.name, name, original, null, [], TraceMethodType.Getter);
        };
        return descriptor;
    }
    return null;
}
function RegisterTraceClass(targetClass) {
    // decorating a class
    // add tracing capability to all own methods (doesn't work for constructor)
    Object.getOwnPropertyNames(targetClass.prototype).forEach((methodName) => {
        const desc = Object.getOwnPropertyDescriptor(targetClass.prototype, methodName);
        // getter
        if (desc === null || desc === void 0 ? void 0 : desc.get) {
            const newDesc = RegisterTraceGetter(targetClass, methodName, desc);
            Object.defineProperty(targetClass, methodName, desc);
            const desc2 = Object.getOwnPropertyDescriptor(targetClass.prototype, methodName);
            //Object.defineProperty = desc;
            return newDesc;
        }
        let original = targetClass.prototype[methodName];
        // constructor
        if (methodName === "constructor") {
            return RegisterTraceValue(targetClass, methodName, desc);
        }
        // skip non functions
        if (typeof original !== "function") {
            return;
        }
        // an arrow function can't be used while we have to preserve right 'this'
        targetClass.prototype[methodName] = function (...args) {
            TraceStub(targetClass.constructor.name, methodName, original, this, args, TraceMethodType.Function);
        };
    });
    return targetClass;
}
function Trace() {
    return (target, name, descriptor) => {
        let trace = RegisterTraceValue(target, name, descriptor);
        if (trace)
            return trace;
        trace = RegisterTraceGetter(target, name, descriptor);
        if (trace)
            return trace;
        return RegisterTraceClass(target);
    };
}
exports.Trace = Trace;
//# sourceMappingURL=trace.js.map